package gregapi.worldgen;

import static gregapi.data.CS.F;
import static gregapi.data.CS.T;
import gregapi.old.GT_Worldgen_Ore;

import java.util.Collection;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

/**
 * @author Gregorius Techneticies
 */
public abstract class Worldgen_Blob extends GT_Worldgen_Ore {
	public Worldgen_Blob(String aName, boolean aDefault, Block aBlock, int aBlockMeta, int aDimensionType, int aAmount, int aSize, int aProbability, int aMinY, int aMaxY, Collection<String> aBiomeList, boolean aAllowToGenerateinVoid) {
		super(aName, aDefault, aBlock, aBlockMeta, aDimensionType, aAmount, aSize, aProbability, aMinY, aMaxY, aBiomeList, aAllowToGenerateinVoid);
	}
	
	@Override
	public boolean executeWorldgen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		if (isGenerationAllowed(aWorld, aDimensionType, mDimensionType) && (mBiomeList.isEmpty() || mBiomeList.contains(aBiome)) && (mProbability <= 1 || aRandom.nextInt(mProbability) == 0)) {
			for (int i = 0; i < mAmount; i++) {
				int tX = aChunkX + aRandom.nextInt(16), tY = mMinY + aRandom.nextInt(mMaxY - mMinY), tZ = aChunkZ + aRandom.nextInt(16);
				if (mAllowToGenerateinVoid || !aWorld.getBlock(tX, tY, tZ).isAir(aWorld, tX, tY, tZ)) {
					float var6 = aRandom.nextFloat() * (float)Math.PI;
					double aX1 = ((tX + 8) + MathHelper.sin(var6) * mSize / 8);
					double aX2 = ((tX + 8) - MathHelper.sin(var6) * mSize / 8);
					double aZ1 = ((tZ + 8) + MathHelper.cos(var6) * mSize / 8);
					double aZ2 = ((tZ + 8) - MathHelper.cos(var6) * mSize / 8);
					double aY1 = (tY + aRandom.nextInt(3) - 2);
					double aY2 = (tY + aRandom.nextInt(3) - 2);
					
					int aMinX = Integer.MAX_VALUE, aMinY = Integer.MAX_VALUE, aMinZ = Integer.MAX_VALUE, aMaxX = Integer.MIN_VALUE, aMaxY = Integer.MIN_VALUE, aMaxZ = Integer.MIN_VALUE;
					
					double[] tRandoms = new double[mSize+1];
					
					for (int j = 0; j < tRandoms.length; j++) tRandoms[j] = aRandom.nextDouble() * mSize / 16;
					
					for (int j = 0; j <= mSize; ++j) {
						double bX = aX1 + (aX2 - aX1) * j / mSize, bY = aY1 + (aY2 - aY1) * j / mSize, bZ = aZ1 + (aZ2 - aZ1) * j / mSize, b = ((MathHelper.sin(j * (float)Math.PI / mSize) + 1) * tRandoms[j] + 1) / 2;
						aMinX = Math.min(aMinX, MathHelper.floor_double(bX - b));
						aMinY = Math.min(aMinY, Math.max(0, MathHelper.floor_double(bY - b)));
						aMinZ = Math.min(aMinZ, MathHelper.floor_double(bZ - b));
						aMaxX = Math.max(aMaxX, MathHelper.floor_double(bX + b));
						aMaxY = Math.max(aMaxY, Math.min(aWorld.getHeight(), MathHelper.floor_double(bY + b)));
						aMaxZ = Math.max(aMaxZ, MathHelper.floor_double(bZ + b));
					}
					
					boolean[][][] tCheck = new boolean[aMaxX-aMinX+1][aMaxY-aMinY+1][aMaxZ-aMinZ+1];
					
					for (int j = 0; j <= mSize; ++j) {
						double bX = aX1 + (aX2 - aX1) * j / mSize, bY = aY1 + (aY2 - aY1) * j / mSize, bZ = aZ1 + (aZ2 - aZ1) * j / mSize, b = ((MathHelper.sin(j * (float)Math.PI / mSize) + 1) * tRandoms[j] + 1) / 2;
						int tMinX = MathHelper.floor_double(bX - b);
						int tMinY = Math.max(0, MathHelper.floor_double(bY - b));
						int tMinZ = MathHelper.floor_double(bZ - b);
						int tMaxX = MathHelper.floor_double(bX + b);
						int tMaxY = Math.min(aWorld.getHeight(), MathHelper.floor_double(bY + b));
						int tMaxZ = MathHelper.floor_double(bZ + b);
						
						for (int eX = tMinX; eX <= tMaxX; ++eX) {
							double cX = (eX + 0.5 - bX) / b;
							if (cX * cX < 1) for (int eY = tMinY; eY <= tMaxY; ++eY) {
								double cY = (eY + 0.5 - bY) / b;
								if (cX * cX + cY * cY < 1) for (int eZ = tMinZ; eZ <= tMaxZ; ++eZ) {
									if (!tCheck[eX-aMinX][eY-aMinY][eZ-aMinZ]) {
										double cZ = (eZ + 0.5 - bZ) / b;
										if (cX * cX + cY * cY + cZ * cZ < 1) {
											tCheck[eX-aMinX][eY-aMinY][eZ-aMinZ] = T;
										}
									}
								}
							}
						}
					}
					
					for (int a = 0; a < tCheck.length; a++) for (int b = 0; b < tCheck[a].length; b++) for (int c = 0; c < tCheck[a][b].length; c++) if (tCheck[a][b][c]) {
						tryPlaceStuff(aWorld, aMinX+a, aMinY+b, aMinZ+c, aRandom);
					}
				}
			}
			return T;
		}
		return F;
	}
	
	public abstract boolean tryPlaceStuff(World aWorld, int aX, int aY, int aZ, Random aRandom);
}